<?php

namespace Drupal\gender\Plugin\Field\FieldWidget;

use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\OptionsSelectWidget;
use Drupal\Core\Form\OptGroup;

/**
 * Plugin implementation of the 'gender_default' widget.
 *
 * @FieldWidget(
 *   id = "gender_default",
 *   label = @Translation("Default"),
 *   field_types = {
 *     "gender"
 *   },
 *   multiple_values = TRUE
 * )
 */
class GenderDefaultWidget extends OptionsSelectWidget {

  /**
   * {@inheritdoc}
   */
  protected function supportsGroups() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  protected function getOptions(FieldableEntityInterface $entity) {
    if (!isset($this->options)) {
      $options = gender_options();

      // Add an empty option if the widget needs one.
      if ($empty_label = $this->getEmptyLabel()) {
        $options = ['_none' => $empty_label] + $options;
      }

      $module_handler = \Drupal::moduleHandler();
      $context = [
        'fieldDefinition' => $this->fieldDefinition,
        'entity' => $entity,
      ];
      $module_handler->alter('options_list', $options, $context);

      array_walk_recursive($options, [$this, 'sanitizeLabel']);

      // Options might be nested ("optgroups"). If the widget does not support
      // nested options, flatten the list.
      if (!$this->supportsGroups()) {
        $options = OptGroup::flattenOptions($options);
      }

      $this->options = $options;
    }

    return $this->options;
  }

}
